@@ -406,13 +406,19 @@ public:
     inline QQmlPropertyData *overrideData(QQmlPropertyData *) const;
     inline bool isAllowedInRevision(QQmlPropertyData *) const;
 
-    static QQmlPropertyData *property(QJSEngine *, QObject *, const QString &,
+    static QQmlPropertyData *property(QJSEngine *, QObject *, const QStringRef &,
                                               QQmlContextData *, QQmlPropertyData &);
     static QQmlPropertyData *property(QJSEngine *, QObject *, const QLatin1String &,
                                       QQmlContextData *, QQmlPropertyData &);
     static QQmlPropertyData *property(QJSEngine *, QObject *, const QV4::String *,
                                               QQmlContextData *, QQmlPropertyData &);
 
+    static QQmlPropertyData *property(QJSEngine *engine, QObject *obj, const QString &name,
+                                      QQmlContextData *context, QQmlPropertyData &local)
+    {
+        return property(engine, obj, QStringRef(&name), context, local);
+    }
+
     //see QMetaObjectPrivate::originalClone
     int originalClone(int index);
     static int originalClone(QObject *, int index);
@@ -441,8 +447,8 @@ public:
     QByteArray checksum(bool *ok);
 
 protected:
-    virtual void destroy();
-    virtual void clear();
+    void destroy() override;
+    void clear() override;
 
 private:
     friend class QQmlEnginePrivate;
@@ -474,7 +480,7 @@ private:
     void updateRecur(const QMetaObject *);
 
     template<typename K>
-    QQmlPropertyData *findNamedProperty(const K &key)
+    QQmlPropertyData *findNamedProperty(const K &key) const
     {
         StringCache::mapped_type *it = stringCache.value(key);
         return it ? it->second : 0;
